@@ -184,7 +184,7 @@ def prepare_binaries(dockerfile_dir, host_directory=None, target_directories=Non
     host_folder_name = Path(host_directory).name
 
     if not target_directories:
-        target_directories = glob(os.path.join(Path(host_directory).parent.absolute(), "target-*", ""))
+        target_directories = glob(os.path.join(Path(host_directory).parent.absolute(), "target-*"))
     target_folder_names = [Path(td).name for td in target_directories]
 
     check_directories(host_directory, target_directories)
