@@ -1,4 +1,4 @@
-# SPDX-FileCopyrightText: Copyright (c) 2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+# SPDX-FileCopyrightText: Copyright (c) 2024-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 # SPDX-License-Identifier: LicenseRef-NvidiaProprietary
 #
 # NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
@@ -29,6 +29,7 @@ class CompositeTable(Enum):
     MPI = 9
     UCX = 10
     GPU_METRICS = 11
+    PERF_EVENTS = 12
 
 
 def get_cuda_gpu_dict():
@@ -276,7 +277,7 @@ def get_nvtx_dict():
 
 
 def process_to_nvtx_table(df_dict):
-    """Generate refined NCCL table.
+    """Generate refined NVTX table.
 
     This function merges the "text" and "textId" fields into a single "text"
     field. Any integer representations in the "text" field are replaced with
@@ -543,6 +544,104 @@ def process_to_gpu_metrics_table(df_dict):
     return gpu_metrics_df.rename(columns=metrics_name_map)
 
 
+def get_perf_events_dict():
+    return {
+        "PERF_EVENT_SOC_OR_CPU_RAW_EVENT": [
+            "start",
+            "end",
+            "vmId",
+            "componentId",
+            "eventId",
+            "count",
+        ],
+        "TARGET_INFO_PERF_METRIC": ["id", "name"],
+        "TARGET_INFO_COMPONENT": ["componentId", "name", "instance", "parentId"],
+    }
+
+
+def process_to_perf_events_table(df_dict):
+    """
+    Generate refined Perf Events table.
+
+    This function merges different tables related to Perf Events data
+    into one to get the refined table.
+    This function:
+    - merges the "TARGET_INFO_COMPONENT" table with itself to get CPU related to the
+        component.
+    - merges the "PERF_EVENT_SOC_OR_CPU_RAW_EVENT" table
+        with the "TARGET_INFO_COMPONENT" to enrich the Perf Events table with
+        the `componentType` and `cpu` columns.
+    - merges the "PERF_EVENT_SOC_OR_CPU_RAW_EVENT" table
+        with the "TARGET_INFO_PERF_METRIC" to enrich the Perf Events table with
+        the `name` column.
+
+    Parameters
+    ----------
+    df_dict : dict
+        Dictionary mapping table names to dataframes. It must contain:
+        -   "PERF_EVENT_SOC_OR_CPU_RAW_EVENT": [
+                "start",
+                "end",
+                "vmId",
+                "componentId",
+                "eventId",
+                "count"
+            ]
+        -   "TARGET_INFO_PERF_METRIC": [
+                "id",
+                "name"
+            ]
+        -   "TARGET_INFO_COMPONENT": [
+                "componentId",
+                "name",
+                "instance",
+                "parentId"
+            ]
+
+    Returns
+    -------
+    Dataframe representing the refined Perf Events table,
+    which contains the following columns:
+        - start: Event start timestamp (ns).
+        - end: Event end timestamp (ns).
+        - vmId: VM ID.
+        - eventId: Event ID.
+        - count: Counter data value.
+        - componentType: Type of the component: Core, Cache, SocketN.
+        - cpu: CPU number.
+        - name: Name of the event.
+    Example:
+    start     end            vmId eventId count componentType cpu                name
+     8912 3655504 281474976710656     113  3382          Core   0        INST_RETIRED
+     8912 3655504 281474976710656     117    29         Cache   0      L2D_TLB_REFILL
+    30224 3657904 281474976710656     4361 3622       Socket0 <NA>   SCF/cmem_rd_data
+    """
+    compnt_df = df_dict["TARGET_INFO_COMPONENT"].astype({"instance": pd.UInt32Dtype()})
+
+    parent_compnt_df = compnt_df[["componentId", "instance"]].rename(
+        columns={"componentId": "parentId", "instance": "parentInstance"}
+    )
+
+    compnt_df = compnt_df.merge(parent_compnt_df, on="parentId", how="left")
+    compnt_df["cpu"] = compnt_df["instance"].combine_first(compnt_df["parentInstance"])
+    compnt_df = compnt_df[["componentId", "name", "cpu"]].rename(
+        columns={"name": "componentType"}
+    )
+
+    perf_metric_df = df_dict["TARGET_INFO_PERF_METRIC"].rename(
+        columns={"id": "eventId"}
+    )
+
+    perf_events_df = (
+        df_dict["PERF_EVENT_SOC_OR_CPU_RAW_EVENT"]
+        .merge(compnt_df, on="componentId", how="left")
+        .merge(perf_metric_df, on="eventId", how="left")
+        .drop(columns=["componentId"])
+    )
+
+    return perf_events_df
+
+
 def update_existing_keys(default_dict, extra_dict):
     """Merge "extra_dict" into "default_dict".
 
@@ -591,6 +690,7 @@ def get_table_column_dict(table, extra_dict=None):
         CompositeTable.MPI: get_mpi_dict,
         CompositeTable.UCX: get_ucx_dict,
         CompositeTable.GPU_METRICS: get_gpu_metrics_dict,
+        CompositeTable.PERF_EVENTS: get_perf_events_dict,
     }
 
     if table not in table_dict_map:
@@ -626,6 +726,7 @@ def get_refine_func(table):
         CompositeTable.MPI: process_to_mpi_table,
         CompositeTable.UCX: process_to_ucx_table,
         CompositeTable.GPU_METRICS: process_to_gpu_metrics_table,
+        CompositeTable.PERF_EVENTS: process_to_perf_events_table,
     }
 
     if table not in table_func_map:
